
#include <windows.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "cdraw.h"

long FAR PASCAL WndProc(HWND      ClientWindow,
                        unsigned  Message,
                        WORD      wparm,
                        LONG      lparm);

char        AppName[] = "EPS File Viewer";
HWND        WinHandle;
HDC         dc;
PAINTSTRUCT ps;
MSG         msg;
WNDCLASS    cdraw_parent;
HANDLE      ThisInstance;
TEXTMETRIC  tm;
RECT        rect;
int         xchar, ychar;
int         phase;
double      points[20];
int         item_count;
WORD        speed;
int         draw_flag;
int         continue_flag;
int         recursion;

int PASCAL WinMain (HANDLE Instance,
                    HANDLE PrevInstance,
                    LPSTR  CmdLine,
                    int    CmdShow)
        {
        ThisInstance = Instance;

        if (!PrevInstance)
            {
            cdraw_parent.style          = CS_HREDRAW | CS_VREDRAW;
            cdraw_parent.lpfnWndProc    = WndProc;
            cdraw_parent.cbClsExtra     = 0;
            cdraw_parent.cbWndExtra     = 0;
            cdraw_parent.hInstance      = Instance;
            cdraw_parent.hIcon          = LoadIcon(Instance, "CDRAW");
            cdraw_parent.hCursor        = LoadCursor(NULL, IDC_ARROW);
            cdraw_parent.hbrBackground  = COLOR_WINDOW + 1;
            cdraw_parent.lpszMenuName   = "CDrawMenu";
            cdraw_parent.lpszClassName  = "CDrawClass";

            if (!RegisterClass(&cdraw_parent))
                return FALSE;
            }

        WinHandle = CreateWindow("CDrawClass", 
                                 AppName,
                                 WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN,
                                 CW_USEDEFAULT,
                                 CW_USEDEFAULT,
                                 CW_USEDEFAULT,
                                 CW_USEDEFAULT,
                                 NULL, NULL, Instance, NULL);
        ShowWindow(WinHandle, CmdShow);
        UpdateWindow(WinHandle);

        while (GetMessage(&msg, NULL, 0, 0))
            {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
            }

        return msg.wParam;
        }

void    LetWindowsBreathe(void)
        {
        MSG   msg;

        if (PeekMessage( (LPMSG) &msg, NULL, 0, 0, PM_REMOVE) )
            if (msg.message != WM_QUIT)
                {
                TranslateMessage( (LPMSG) &msg);
                DispatchMessage( (LPMSG) &msg);
                }
        }

void    sleep(int secs)
        {
        long now, alarm;

        time(&now);
        alarm = now + (long) secs;
        while (now < alarm)
            {
            LetWindowsBreathe();
            time(&now);
            }
        }

void    draw_curve (HDC dc,
                    int x1, int y1,
                    int x2, int y2,
                    int x3, int y3,
                    int x4, int y4)
        {
        MoveTo(dc, x1, y1);
        LineTo(dc, x2, y2);
        LineTo(dc, x3, y3);
        LineTo(dc, x4, y4);
        }

void    deCasteljau(HDC dc,
                    double x1, double y1,
                    double x2, double y2,
                    double x3, double y3,
                    double x4, double y4)
        {
        double xa1, ya1;
        double xa2, ya2;
        double xa3, ya3;
        double xb1, yb1;
        double xb2, yb2;
        double xc1, yc1;

        if (recursion >= 3)
            {
            draw_curve(dc, (int) x1, (int) y1,
                           (int) x2, (int) y2,
                           (int) x3, (int) y3,
                           (int) x4, (int) y4);
            return;
            }

        recursion++;

        xa1 = x1  + (x2  - x1)  / 2;
        ya1 = y1  + (y2  - y1)  / 2;

        xb1 = x2  + (x3  - x2)  / 2;
        yb1 = y2  + (y3  - y2)  / 2;

        xa2 = xa1 + (xb1 - xa1) / 2;
        ya2 = ya1 + (yb1 - ya1) / 2;

        xc1 = x3  + (x4  - x3)  / 2;
        yc1 = y3  + (y4  - y3)  / 2;

        xb2 = xb1 + (xc1 - xb1) / 2;
        yb2 = yb1 + (yc1 - yb1) / 2;

        xa3 = xa2 + (xb2 - xa2) / 2;
        ya3 = ya2 + (yb2 - ya2) / 2;

        deCasteljau (dc, x1, y1, xa1, ya1, xa2, ya2, xa3, ya3);
        deCasteljau (dc, xa3, ya3, xb2, yb2, xc1, yc1, x4, y4);

        recursion--;
        }

void    ShowEPS(void)
        {
        HPEN            hPen;
        FILE            *textfile;
        DWORD           curr_point;
        char            textline[81];
        char            *p;
        short           x, y;

        textfile = fopen("MUSIC.EPS", "r");
        if (textfile == NULL)
            {
            MessageBox( NULL,
                        "Error.  Could not open EPS file MUSIC.EPS.",
                        AppName,
                        MB_OK | MB_ICONHAND );
            return;
            }

        hPen = SelectObject( dc, CreatePen(PS_SOLID, 1, 0l) );
        SetMapMode(dc, MM_HIENGLISH);
        SetViewportOrg(dc, 0, rect.bottom);
        item_count = 0;

        while (fgets(textline, 80, textfile) != NULL)
            {
            if (!continue_flag)
                break;
            textline[80] = '\0';
            strtok(textline, "\n");
            if (textline[0] == ';') continue;
            if (textline[0] == '*') continue;
            if (strlen(textline) < 3) continue;
            LetWindowsBreathe();
            item_count = 0;
            p = strtok(textline, " ");
            while (p != NULL && *p != 'm' && *p != 'l' && *p != 'c')
                {
                if (item_count > 18)
                    {
                    p = NULL;
                    break;
                    }
                points[item_count++] = atof(p);
                p = strtok(NULL, " ");
                }
            if (p == NULL) continue;
            if (speed == IDM_SLOW)
                sleep(1);
            switch (*p)
                {
                case 'm':
                    x = points[0] * 10;
                    y = points[1] * 8;
                    MoveTo(dc, x, y);
                    break;
                case 'l':
                    x = points[0] * 10;
                    y = points[1] * 8;
                    LineTo(dc, x, y);
                    break;
                case 'c':
                    curr_point = GetCurrentPosition(dc);
                    points[6] = (double) LOWORD(curr_point);
                    points[7] = (double) HIWORD(curr_point);
                    deCasteljau(dc,
                                points[6],      points[7],
                                points[0] * 10, points[1] * 8,
                                points[2] * 10, points[3] * 8,
                                points[4] * 10, points[5] * 8);
                    break;
                default:
                    break;
                }
            }

        fclose(textfile);
        DeleteObject(hPen);
        return;
        }

long FAR PASCAL WndProc(HWND      ClientWindow,
                        unsigned  Message,
                        WORD      wparm,
                        LONG      lparm)
        {
        switch (Message)
            {
            case WM_CREATE:
                dc = GetDC(ClientWindow);
                GetTextMetrics(dc, &tm);
                xchar = tm.tmAveCharWidth;
                ychar = tm.tmHeight + tm.tmExternalLeading;
                ReleaseDC(ClientWindow, dc);
                break;

            case WM_SIZE:
                rect.left   = 2 * xchar;
                rect.top    = 2 * ychar;
                rect.right  = LOWORD(lparm);
                rect.bottom = HIWORD(lparm);
                break;

            case WM_PAINT:
                dc = BeginPaint(ClientWindow, &ps);
                if (draw_flag)
                    ShowEPS();
                EndPaint(ClientWindow, &ps);
                draw_flag = FALSE;
                break;

            case WM_LBUTTONUP:
                continue_flag = FALSE;
                break;

            case WM_COMMAND:
                if (wparm == IDM_FAST || wparm == IDM_SLOW)
                    {
                    speed = wparm;
                    break;  
                    }
                if (wparm == IDM_DRAW)
                    {
                    continue_flag = TRUE;
                    draw_flag = TRUE;
                    InvalidateRect(ClientWindow, NULL, TRUE);
                    break;
                    }
                if (wparm == IDM_QUIT)
                    {
                    DestroyWindow(ClientWindow);
                    break;
                    }

            case WM_DESTROY:
                PostQuitMessage(0);
                break;

            default:
                return DefWindowProc(ClientWindow, Message, wparm, lparm);
            }

        return 0l;
        }

